
#include <math.h>
#ifndef sun
#define FLOATFUNCTIONTYPE float
#define RETURNFLOAT return
#endif

/*****************************************************************************/
/*                                                                           */
/* Routines:       srand / rand / ran                                      */
/*                                                                           */
/* Purpose:        Provide a consistent 32-bit random number generator       */
/*                 no matter what machine the code is running on             */
/*                                                                           */
/* Revision:       0 : Initial Documented Version, 3/88, J. Brown, DLM       */
/*                 1 : Created _ranto correspond to Convex RAN(), 8/89, DLM */
/*                                                                           */
/* Usage:          int irand = srand( seed )                                 */
/*                 int seed;    (I: Initial seed for random number generator)*/
/*                                                                           */
/*                 int irand = rand()                                        */
/*                                                                           */
/*                 float frand = _ran_( pseed )                               */
/*                 int *pseed;  (I/O: seed used for generator)               */
/*                                                                           */
/* Notes:          rand() returns 32-bit random numbers evenly distributed   */
/*                 in the range 0 -> 2**32 - 1.  This utility was created    */
/*                 because various C compilers (A T & T) Unix, Berkeley Unix,*/
/*                 DEC) cannot agree on whether to provide srand/rand with   */
/*                 16-bit or 32-bit  capability or which coefficients to use */
/*                 For _ran_(), the number returned is scaled to              */
/*                 unity.                                                    */ 
/*                 Sun note:  returning a float must be handled specially    */
/*                                                                           */
/*****************************************************************************/

#define RANDMULT 1103515245
#define RANDADD  12345
#define RANDMASK (0x7fffffff)

static long randx;


/*  srand() allows the seed to be reset; still returns a random number */

int srand(seed)
int seed;
{ 
  randx = seed;
  randx = ( RANDMULT*randx + RANDADD ) & RANDMASK;
  return(randx);
}


/*  rand() uses a seed which is already set and returns a random number */

int rand()
{
  randx = ( RANDMULT*randx + RANDADD ) & RANDMASK;
  return(randx);
}


/*  _ran_() returns a float between 0 and 1 plus the new value of the seed  */
/*  (Fortran-callable)                                                     */

FLOATFUNCTIONTYPE ran( pseed )
int *pseed;
{
  static int irmax = 2147483647;
  float r;
  randx = *pseed;
  randx = ( RANDMULT*randx + RANDADD ) & RANDMASK;
  r = (float)randx/(float)irmax;
  *pseed = randx;
  RETURNFLOAT ( r );
}


